/**************************************************************************/
/* tg_utils.c - Utility functions for the "Trek    Guru" program.         */
/*                                                                        */
/*                Original creator: Rob Seace (ras@world.std.com)         */
/*                                                                        */
/*        Note: If you make ANY changes at all to any of the code, PLEASE */
/*              make note of it with comments, as well as in the "README" */
/*              file, especially if re-distributing the new code.         */
/**************************************************************************/


#include "trekguru.h"


/* tg_menu - Displays a menu of choices to the user, and gets a choice (int)
             from them.  If the int is one of the valid choices, it is
             returned to the calling function, else it requests a valid choice.
             The parameter of the function is a list of text strings to
             display for the choices (terminated with a NULL).  The first
             string is choice 0, the next is 1, and so on...
*/

int tg_menu (choice_strs)
    char    *choice_strs[];          /* The choices (text to display)    */
{
    int        choice;               /* The user's entered choice        */
    int        i;                    /* Loop counter                     */
    char    buf[MAX_BUF_LEN+1];      /* input buffer                     */

    printf ("\f\n\n");    /* "\f" is formfeed == clear screen */

    for (i = 0; choice_strs[i] ISNT NULL; i++)
        printf ("    %2d -- %s\n", i, choice_strs[i]);

    for (;;) {
        printf ("\n\nEnter your choice [0 - %d]: ", i-1);

        fgets (buf, MAX_BUF_LEN, stdin);

        if (sscanf (buf, "%d", &choice) ISNT 1)
            printf ("\07\nInvalid choice: %s\n", buf);
        else if ((choice >= 0) AND (choice < i))
            return (choice);
        else
            printf ("\07\nInvalid choice: %d\n", choice);
    }
}


/* tg_help_main - Displays help info for main menu.
*/

void tg_help_main ()
{
    char    junk[MAX_BUF_LEN+1];      /* a junk buffer for reading input    */

    printf ("\f\n\n");

    printf ("Choice 0: Exits the program, and returns you to your OS.\n\n");

    printf ("Choice 1: Display this help screen.\n\n");

    printf ("Choice 2: Loads only TOS (The Original Series) information.\n");
    printf ("          Use this if you want to find info relating only to TOS.\n\n");

    printf ("Choice 3: Loads only TAS (The Animated Series) information.\n");
    printf ("          Use this if you want to find info relating only to TAS.\n\n");

    printf ("Choice 4: Loads only TMS (The Movie Series) information.\n");
    printf ("          Use this if you want to find info relating only to TMS.\n\n");

    printf ("Choice 5: Loads only TNG (The Next Generation) information.\n");
    printf ("          Use this if you want to find info relating only to TNG.\n\n");

    printf ("Choice 6: Loads only DS9 (Deep Space Nine) information.\n");
    printf ("          Use this if you want to find info relating only to DS9.\n\n");

    printf ("\nPress Enter to continue.");
    fgets (junk, MAX_BUF_LEN, stdin);

    printf ("\f\n\n");

    printf ("Choice 7: Loads all of the information, on all series available.\n");
    printf ("          Use this if you want to find info relating to a combination\n");
    printf ("          of the series (for instance: to see what roles an actor has\n");
    printf ("          played in any of the series).\n\n");

    printf ("Note: At the time this program was written, neither TNG nor DS9 info\n");
    printf ("      was yet available.  So, selecting choice 5 or 6 will cause the\n");
    printf ("      program to display an error message explaining that the data\n");
    printf ("      files can't be found.  And, selecting choice 7, will load only\n");
    printf ("      those files that are available.  The reason I put the TNG and\n");
    printf ("      DS9 selections in now, is to make it easier for when I do release\n");
    printf ("      the TNG and DS9 info (that is whenever I can ever find the time\n");
    printf ("      to enter all that information).  This way, I need not bother to\n");
    printf ("      change and re-distribute the code for this program again, just the\n");
    printf ("      new data files.  (Note: In +f mode, no error message will be\n");
    printf ("      displayed on choices 5 or 6.  There will simply be no data to show.)\n\n");

    printf ("\nPress Enter to continue.");
    fgets (junk, MAX_BUF_LEN, stdin);

    printf ("\f\n\n");

    printf ("Command Line Usage:  trekguru [?] [+f] [d=<dir>]\n\n");

    printf ("                      ?  -  Prints out a usage summary.\n\n");

    printf ("                     +f  -  Causes the program to use file access for all\n");
    printf ("                            searching.  Normally, the program loads the\n");
    printf ("                            info into memory, and does searches in memory.\n");
    printf ("                            Only use this option if you are low on memory.\n");
    printf ("                            Obviously, searches become significantly slower.\n\n");

    printf ("                  d=<dir> - Sets the directory for the data files, if they\n");
    printf ("                            are not in the current directory.\n\n");

    printf ("\nPress Enter to continue.");
    fgets (junk, MAX_BUF_LEN, stdin);
}


/* tg_help_sub - Displays help info for sub-menu.
*/

void tg_help_sub ()
{
    char    junk[MAX_BUF_LEN+1];      /* a junk buffer for reading input    */

    printf ("\f\n\n");

    printf ("Choice 0: Return back to the Main Menu.\n\n");

    printf ("Choice 1: Show this help information.\n\n");

    printf ("Choice 2: Searches through the actor database(s) for the selected series,\n");
    printf ("          for the actor(s) who matches your input name, and displays the\n");
    printf ("          full actor/character information for any matches found.\n\n");

    printf ("Choice 3: Same as Choice 2, but you must enter a character name, instead\n");
    printf ("          of an actor name.\n\n");

    printf ("Choice 4: Same as Choice 2, but you must enter an episode production number.\n");
    printf ("          NOTE: This is not a very reliable search method, since, due to\n");
    printf ("          space limitations, I couldn't store EVERY episode that EVERY\n");
    printf ("          actor/character appeared in.  For those who appeared in a very\n");
    printf ("          large number of episodes (eg: Kirk), I had to abbreviate their\n");
    printf ("          episode info (eg: 1-79), rather than list each and every one.\n");
    printf ("          However, I don't consider this a major problem, as you likely\n");
    printf ("          wouldn't be searching for one of these major actors/characters\n");
    printf ("          by this method, anyway, since you know they are in practically\n");
    printf ("          every episode.  But, for minor actors, it works fine.\n\n");

    printf ("Press Enter to continue.");
    fgets (junk, MAX_BUF_LEN, stdin);

    printf ("\f\n\n");

    printf ("Choice  5: Searches through the episode database(s) for the selected series,\n");
    printf ("           for a Title which matches the input string, and displays the full\n");
    printf ("           episode information for any mathches found.\n\n");

    printf ("Choice  6: Same as Choice 5, but you must enter a Production Number to search\n");
    printf ("           for, instead of a title.\n\n");

    printf ("Choice  7: Same as Choice 5, but you must enter an Original Airdate.\n\n");

    printf ("Choice  8: Same as Choice 5, but you must enter a Stardate.\n\n");

    printf ("Choice  9: Same as Choice 5, but you must enter a Writer's name.\n\n");

    printf ("Choice 10: Same as Choice 5, but you must enter a Director's name.\n\n");

    printf ("Choice 11: Same as Choice 5, but you must enter a keyword from the episode's\n");
    printf ("           Synopsis.  This is a very useful search method.  It allows you to\n");
    printf ("           find an episode based on whatever feature you can remember about\n");
    printf ("           it.  For instance, if you wanted to find the episode that had the\n");
    printf ("           Space Hippies in it, just type in 'hippie' as the search word, and\n");
    printf ("           program would find the info for the episode: 'The Way to Eden'.\n\n");

    printf ("Press Enter to continue.");
    fgets (junk, MAX_BUF_LEN, stdin);

    printf ("\f\n\n");

    printf ("Choice 12: Searches through the episode database(s) for the selected series,\n");
    printf ("           for an episode(s) which matches all of the input search criteria.\n");
    printf ("           Any or all of the previously mentioned search criteria can be\n");
    printf ("           entered, to use in the search.\n\n");

    printf ("Choice 13: Finds and displays the full information for all of the actors and\n");
    printf ("           characters in the currently selected actor database(s).\n\n");

    printf ("Choice 14: Finds and displays the full information for all of the episodes in\n");
    printf ("           the currently selected episode database(s).\n\n");

    printf ("Choice 15: Displays the names of all of the actors and characters (one line\n");
    printf ("           for each actor/character pair) in the selected actor database(s).\n\n");

    printf ("Choice 16: Displays the titles and episode numbers of all the episodes (one\n");
    printf ("           line for each title/number) in the selected episode database(s).\n\n");

    printf ("Press Enter to continue.");
    fgets (junk, MAX_BUF_LEN, stdin);

    printf ("\f\n\n");

    printf ("Note: All matching is substring type.  Eg: 'the' would match to 'The Ultimate\n");
    printf ("      Computer', as well as 'Return of the Archons' (among many others), if\n");
    printf ("      entered as an episode title.  Also, case is not important.  Eg: 'spock'\n");
    printf ("      will match 'Spock' and 'SPOCK' and 'SpOcK', etc.\n\n\n");


    printf ("Here is the episode numbering scheme used in this program:\n\n");

    printf ("    TOS: All episode numbers are just numbers, 1 - 79.\n");
    printf ("    TAS: All episode numbers are numbers, 1 - 22, followed immediately by the\n");
    printf ("         letter 'A' (eg: '4A').\n");
    printf ("    TMS: All episode numbers are Roman numerals, I - VI (for now...).\n");
    printf ("    TNG: All episode numbers are just numbers, 101 - ...\n");
    printf ("    DS9: All episode numbers are just numbers, 401 - ...\n\n");

    printf ("Press Enter to continue.");
    fgets (junk, MAX_BUF_LEN, stdin);
}


/* tg_streq - Determines if the 2 passed strings are "equal"; ie: whether
                they match or not.  To match, the srch_str must be either
                empty (except possibly for a newline), or be a substring
                of the db_str.  The db_str_len is the length of this database
                string field (so that I can ensure null-termination, since
                there likely won't be NULLs in the datafile).  The function
                returns TRUE on a match, else FALSE.
        Note: Case is unimportant.  Both strings are converted to upper
                case before comparison.
*/

int tg_streq (srch_str, db_str, db_str_len)
    char    *srch_str;        /* the user-entered search string            */
    char    *db_str;          /* the database string to compare it with    */
    int     db_str_len;       /* the length of this database string field  */
{
    char    my_db_str[MAX_BUF_LEN+1];  /* temp copy of db_str               */
    int     i;                /* loop counter                               */
    int     len;              /* length of srch_str                         */

    if ((srch_str IS NULL) OR (db_str IS NULL) OR (db_str_len IS 0))
        return (TRUE);        /* null, 0-length strings; must be find-all */

    if ((srch_str[0] IS '\0') OR (srch_str[0] IS '\n'))
        return (TRUE);        /* empty string matches anything */

    len = strlen (srch_str);

    if (len > db_str_len)
        return (FALSE);        /* can't be a substring if len > db_str_len */

    for (i = 0; i < len; i++) {
                            /* in case there is no strupper() or toupper() */
        if ((srch_str[i] >= 'a') AND (srch_str[i] <= 'z'))
            srch_str[i] = (srch_str[i] - ('a' - 'A'));
    }

    strncpy (my_db_str, db_str, db_str_len);
    my_db_str[db_str_len] = '\0';

    for (i = 0; i < db_str_len; i++) {
                            /* in case there is no strupper() or toupper() */
        if ((my_db_str[i] >= 'a') AND (my_db_str[i] <= 'z'))
            my_db_str[i] = (my_db_str[i] - ('a' - 'A'));
    }

    for (i = 0; (i + len) <= db_str_len; i++) {
        if (NOT strncmp (srch_str, &(my_db_str[i]), len))
            return (TRUE);
    }

    return (FALSE);        /* if we get out of last loop, then not a substr */
}


/* tg_add_dir - Adds the input data directory, in DatDir, to the beginning
                of the passed filename, returning the resultant string.
*/

char *tg_add_dir (fname)
    char    *fname;                           /* the file name              */
{
static    char    full_fname[MAX_BUF_LEN+1];  /* full file name, with dir   */

    full_fname[0] = '\0';

    if ((DatDir ISNT NULL) AND (DatDir[0] ISNT '\0')) {
        strcpy (full_fname, DatDir);
        if (DatDir[strlen (DatDir) - 1] ISNT '/')
            strcat (full_fname, "/");
    }

    strcat (full_fname, fname);

    return (full_fname);
}

